@@ -9,8 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2008 University of Houston. All rights reserved.
+ * Copyright (c) 2007-2009 University of Houston. All rights reserved.
  * Copyright (c) 2007-2008 Cisco, Inc. All rights reserved.
+ * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -349,7 +350,7 @@ int ompi_comm_create ( ompi_communicator_t *comm, ompi_group_t *group,
              newcomp->c_contextid, comm->c_contextid );
 
     /* Activate the communicator and init coll-component */
-    rc = ompi_comm_activate( &newcomp );  /* new communicator */ 
+    rc = ompi_comm_activate( &newcomp, 1 );  /* new communicator */ 
     if ( OMPI_SUCCESS != rc ) {
         goto exit;
     }
@@ -572,7 +573,17 @@ int ompi_comm_split ( ompi_communicator_t* comm, int color, int key,
              newcomp->c_contextid, comm->c_contextid );
 
     /* Activate the communicator and init coll-component */
-    rc = ompi_comm_activate( &newcomp );  /* new communicator */ 
+    if ( MPI_UNDEFINED != color ) {
+	rc = ompi_comm_activate( &newcomp, 1 );  /* new communicator */ 
+    }
+    else {
+	/* 
+        ** this should prevent creating the subcommunicators
+        ** of the hierarch module for the cases where we
+        ** know that they will be freed anyway (e.g. color = MPI_UNDEFINED
+	*/
+	rc = ompi_comm_activate( &newcomp, 0 );  /* new communicator */ 
+    }
     if ( OMPI_SUCCESS != rc ) {
         goto exit;
     }
@@ -663,7 +674,7 @@ int ompi_comm_dup ( ompi_communicator_t * comm, ompi_communicator_t **newcomm )
              newcomp->c_contextid, comm->c_contextid );
 
     /* activate communicator and init coll-module */
-    rc = ompi_comm_activate( &newcomp );  /* new communicator */ 
+    rc = ompi_comm_activate( &newcomp, 1 );  /* new communicator */ 
     if ( OMPI_SUCCESS != rc ) {
         return rc;
     }
@@ -959,7 +970,7 @@ int ompi_comm_free ( ompi_communicator_t **comm )
        COMM_NULL.  This is according to MPI-2:88-89. */
 
     if (*comm == ompi_mpi_comm_parent && comm != &ompi_mpi_comm_parent) {
-        ompi_mpi_comm_parent = &ompi_mpi_comm_null;
+        ompi_mpi_comm_parent = &ompi_mpi_comm_null.comm;
     }
 
     /* Release the communicator */
@@ -1500,13 +1511,17 @@ int ompi_topo_create (ompi_communicator_t *old_comm,
         return ret;
     }
 
-    ret = ompi_comm_activate( &new_comm );  /* new communicator */
+    if ( MPI_UNDEFINED != new_rank ) {
+	ret = ompi_comm_activate( &new_comm, 1 );  /* new communicator */
+    }
+    else {
+	ret = ompi_comm_activate( &new_comm, 0 );  /* new communicator */
+    }
     if (OMPI_SUCCESS != ret) {
-        /* something wrong happened during setting the communicator */
-        *comm_topo = new_comm;
-        return ret;
+	/* something wrong happened during setting the communicator */
+	*comm_topo = new_comm;
+	return ret;
     }
-
     
     /* if the returned rank is -1, then this process is not in the 
      * new topology, so free everything we have allocated and return */
