@@ -38,3 +38,15 @@
 # define vt_fnmatch(_pattern, _string, __flags) strcmp(_string, _pattern)
 #endif /* HAVE_FNMATCH_H */
 
+#include <limits.h>
+#if defined(HAVE_SYS_PARAM_H) && HAVE_SYS_PARAM_H
+# include <sys/param.h>
+#endif
+#if defined(PATH_MAX)
+# define VT_PATH_MAX (PATH_MAX + 1)
+#elif defined(_POSIX_PATH_MAX)
+# define VT_PATH_MAX (_POSIX_PATH_MAX + 1)
+#else
+# define VT_PATH_MAX 256
+#endif
+
