@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
@@ -24,7 +24,15 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -373,6 +381,26 @@ static int save_filename(const char *filename, lt_ptr data)
 }
 
 
+static int file_exists(const char *filename, const char *ext)
+{
+    char *final;
+    struct stat buf;
+    int ret;
+
+    if (NULL != ext) {
+        asprintf(&final, "%s.%s", filename, ext);
+    } else {
+        final = strdup(filename);
+    }
+    if (NULL == final) {
+        return 0;
+    }
+    ret = stat(final, &buf);
+    free(final);
+    return (0 == ret ? 1 : 0);
+}
+
+
 /*
  * Open a component, chasing down its dependencies first, if possible.
  */
@@ -442,6 +470,20 @@ static int open_component(component_file_item_t *target_file,
 #endif
   if (NULL == component_handle) {
     err = strdup(lt_dlerror());
+    /* Because libltdl erroneously says "file not found" for any type
+       of error -- which is especially misleading when the file is
+       actually there but cannot be opened for some other reason
+       (e.g., missing symbol) -- do some simple huersitics and if the
+       file [probably] does exist, print a slightly better error
+       message. */
+    if (0 == strcmp("file not found", err) &&
+        (file_exists(target_file->filename, "lo") ||
+         file_exists(target_file->filename, "so") ||
+         file_exists(target_file->filename, "dylib") ||
+         file_exists(target_file->filename, "dll"))) {
+        free(err);
+        err = strdup("perhaps a missing symbol, or compiled for a different version of Open MPI?");
+    }
     opal_output_verbose(vl, 0, "mca: base: component_find: unable to open %s: %s (ignored)", 
                         target_file->filename, err);
     free(err);
@@ -506,6 +548,22 @@ static int open_component(component_file_item_t *target_file,
     return OPAL_ERR_BAD_PARAM;
   }
 
+  /* Also check that the component struct framework and component
+     names match the expected names from the filename */
+  if (0 != strcmp(component_struct->mca_type_name, target_file->type) ||
+      0 != strcmp(component_struct->mca_component_name, target_file->name)) {
+      opal_output_verbose(vl, 0, "Component file data does not match filename: %s (%s / %s) != %s %s -- ignored",
+                          target_file->filename, target_file->type, target_file->name,
+                          component_struct->mca_type_name, 
+                          component_struct->mca_component_name);
+      free(mitem);
+      free(struct_name);
+      lt_dlclose(component_handle);
+      target_file->status = FAILED_TO_LOAD;
+      free_dependency_list(&dependencies);
+      return OPAL_ERR_BAD_PARAM;
+  }
+
   /* Alles gut.  Save the component struct, and register this
      component to be closed later. */
 
