@@ -752,8 +752,15 @@ static void job_completed(int trigpipe, short event, void *arg)
                             orte_max_timeout, timeout_callback);
     }
     
+#ifndef __WINDOWS__
     /* now wait to hear it has been done */
     opal_event_dispatch();
+#else
+    /* We are using WT_EXECUTEINWAITTHREAD mode of threading pool,
+       the other callbacks won't be triggerred until this thread finishes,
+       so just return to main thread and process the rest events there.  */
+    return;
+#endif
     
     /* if we cannot order the daemons to terminate, then
      * all we can do is cleanly exit ourselves
@@ -1015,6 +1022,14 @@ static void abort_exit_callback(int fd, short ign, void *arg)
         fprintf(stderr, "%s: killing job...\n\n", orterun_basename);
     }
     
+    /* since we are being terminated by a user's signal, be
+     * sure to exit with a non-zero exit code - but don't
+     * overwrite any error code from a proc that might have
+     * failed, in case that is why the user ordered us
+     * to terminate
+     */
+    ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
+
     /* terminate the job - this will also wakeup orterun so
      * it can report to the user and kill all the orteds.
      * Check the jobid, though, just in case the user
@@ -1065,7 +1080,7 @@ static void abort_exit_callback(int fd, short ign, void *arg)
 
         orte_finalize();
         free(orterun_basename);
-        ORTE_UPDATE_EXIT_STATUS(1);
+        ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
         exit(orte_exit_status);
     }
 }
